//
// File:        RelativiseNamesVisitor.java
// Package:     gov.llnl.babel.parsers.xml2
// Copyright:   (c) 2006 The Regents of the University of California
// Release:     $Name$
// Revision:    @(#) $Revision: 5295 $
// Date:        $Date: 2006-03-24 14:02:42 -0800 (Fri, 24 Mar 2006) $
// Description: AST visitor to relativise names
// 

package gov.llnl.babel.parsers.xml2;

import gov.llnl.babel.ast.ASTNode;
import gov.llnl.babel.ast.ClassType;
import gov.llnl.babel.ast.Enumeration;
import gov.llnl.babel.ast.InterfaceType;
import gov.llnl.babel.ast.Name;
import gov.llnl.babel.ast.NamedType;
import gov.llnl.babel.ast.Package;
import gov.llnl.babel.visitor.Visitor;

public class RelativiseNamesVisitor extends Visitor {
  private String normalizeName(String name) {
    if (name.startsWith(".")) {
      return name.substring(1);
    }
    return name;
  }

  private Object visitNamedType(NamedType node, Object data) {
    ASTNode parent = node.getParent();
    if ((parent != null) && (parent instanceof NamedType)) {
      NamedType namedParent = (NamedType)parent;
      String parentName = normalizeName(namedParent.getFQN()) + ".";
      String myName = normalizeName(node.getName().toString());
      if (myName.startsWith(parentName)) {
        node.setName(new Name(myName.substring(parentName.length())));
      }
    }
    return data;
  }

  public Object visitClassType(ClassType node, Object data) {
    return visitNamedType(node, data);
  }

  public Object visitInterfaceType(InterfaceType node, Object data) {
    return visitNamedType(node, data);
  }

  public Object visitPackage(Package node, Object data) {
    super.visitPackage(node, data);
    return visitNamedType(node, data);
  }

  public Object visitEnumeration(Enumeration node, Object data) {
    return visitNamedType(node, data);
  }
}
