//
// File:        AddDefaultsVisitor.java
// Package:     
// Copyright:   (c) 2006 The Regents of the University of California
// Release:     $Name$
// Revision:    @(#) $Revision: 5496 $
// Date:        $Date: 2006-05-18 21:34:19 -0700 (Thu, 18 May 2006) $
// Description: 
// 
package gov.llnl.babel.visitor;

import gov.llnl.babel.BabelConfiguration;
import gov.llnl.babel.msg.MsgList;
import gov.llnl.babel.ast.ClassType;
import gov.llnl.babel.ast.InterfaceType;
import gov.llnl.babel.ast.ScopedID;
import gov.llnl.babel.ast.Method;
import gov.llnl.babel.ast.ImplementsList;

import java.util.ListIterator;
/**
 * Iterates over a AST a second time to decorate a primed SymbolTable
 * with all the details about classes, interfaces, and structs
 * 
 * Type resolution is largely performed in this stage.
 * 
 * @see gov.llnl.babel.visitor.SymbolTablePrimer
 * @author kumfert
 */
public class AddDefaultsVisitor  extends Visitor {
  
  private final static String SIDL_CLASS
    = "." + BabelConfiguration.getBaseClass();
  
  private final static String SIDL_INTERFACE 
    = "." + BabelConfiguration.getBaseInterface();
  
  private final static String SIDL_IMPLICIT_THROW 
    = "." + BabelConfiguration.getRuntimeException();

  protected MsgList d_msgs;

  public AddDefaultsVisitor(MsgList msgs) {
    super();
    d_msgs = msgs;
  }
  
  /** Add defaults at the Class Level.  
   *  1. If the class is SIDL_BASECLASS, implement SIDL_BASEINTERFACE
   *  2. If the class doesn't inherit anything, extend SIDL_BASECLASS
   */
  public Object visitClassType(ClassType node, Object data) { 
    if (node.getFQN().equals(SIDL_CLASS)) {
      if(node.getImplementsList().size() == 0) { 
        ImplementsList implementsList = node.getImplementsList();
        implementsList.add(new ScopedID(SIDL_INTERFACE));
      }
    } else if ( node.getExtends() == null ) {
      node.setExtends(new ScopedID(SIDL_CLASS));
    }

    Object result = super.visitClassType(node,data); //Visit children
    return result;
  }
  
  /** Add defaults as the interface level
   *  1. If it's not SIDL_BASEINTERFACE, and it has no extends, extend
   *     SIDL_BASEINTERFACE
   */
  public Object visitInterfaceType(InterfaceType node, Object data) { 
    if (!node.getFQN().equals(SIDL_INTERFACE) && node.getExtends().size() == 0) {
      node.getExtends().add(new ScopedID(SIDL_INTERFACE));
    }
    
    //Visit children
    Object result = super.visitInterfaceType(node,data);
    return result;    
  }


  /** Method level defaults
   *  1. Add implicit Runtime Exception to every throws list
   */
  public Object visitMethod(Method node, Object data) { 
    ListIterator i_thrws = node.getThrowsList().listiterator();
    boolean has_implicit_ex = false;
    for(; i_thrws.hasNext();) {
      ScopedID id = (ScopedID) i_thrws.next();
      if(id.toString().equals(SIDL_IMPLICIT_THROW)) {
        has_implicit_ex = true;
      }
    }
    if(!has_implicit_ex) {
      i_thrws.add(new ScopedID(SIDL_IMPLICIT_THROW));
    }
    Object result = super.visitMethod(node,data);  //Visit children
    return result;
  }
}
